/*
 * Graph.hpp
 *
 *  Created on: Jul 16, 2009
 *      Author: kottlers
 */

#ifndef GRAPH_HPP_
#define GRAPH_HPP_

#include <vector>
#include "IFStream.hpp"
#include <cassert>

using std::vector;

struct Vertex{
	uint id;
	Vertex(uint i):id(i){}
};

class Graph{
	uint edge_count;
	vector<vector<Vertex> > adjacency;

protected:
	void ensureVertex(const Vertex& v){
		if(v.id > adjacency.size()){adjacency.resize(v.id);}
	}

	void addEdge(const Vertex& a, const Vertex& b){
		ensureVertex(a);
		ensureVertex(b);
		getAdjaOfVertex(a).push_back(b);
		getAdjaOfVertex(b).push_back(a);
		//printf(" %d  %d \n",a.id,b.id);
		++edge_count;
	}

public:
	uint parsed_grid_width;
	uint parsed_grid_height;

	vector<Vertex>& getAdjaOfVertex(const Vertex& v){
		uint idx = v.id-1;
		assert(idx < adjacency.size());
		return adjacency[idx];
	}

	Graph():edge_count(0),parsed_grid_width(0),parsed_grid_height(0){}
	inline bool parseTGF(IFStream& in);
	uint vertexCount(){	return adjacency.size();}
	uint edgeCount(){	return edge_count;}
};


bool Graph::parseTGF(IFStream& in){
	uint source, target;
	uint max = 0;
	if(in.hasNext() && in.currEquals('g')) {
	   ++in;
	   parsed_grid_width 	= in.getNextInt();
	   parsed_grid_height 	= in.getNextInt();
	   in.skipLine();
	}

    while(in.hasNext()){
    	if(in.currEquals('#')) {in.skipLine();break;}
    	source = in.getNextInt();
    	max = source > max ? source:max;
    	in.skipLine();
    }
    ensureVertex(Vertex(max));

    while(in.hasNext()){
    	source = in.getNextInt();
    	if(source == 0) source = max;

    	if(!in.hasNext()){
    		printf("TGF File contains strange edge with source %d and no target \n",source);
    		return false;
    	}

    	target = in.getNextInt();
    	if(target == 0) target = max;
    	addEdge(Vertex(source),Vertex(target));

    	in.skipLine();
	}
    return true;
}








#endif /* GRAPH_HPP_ */
